@@ -45,13 +45,6 @@ const int polygonPointSize=3;
 /** maximal distance for selection of one point */
 const int maxSelectionDistance=20;
 
-#if !wxCHECK_VERSION(3,0,0)
-#define wxPENSTYLE_SOLID wxSOLID
-#define wxPENSTYLE_DOT wxDOT
-#define wxBRUSHSTYLE_TRANSPARENT wxTRANSPARENT
-#define wxBRUSHSTYLE_SOLID wxSOLID
-#endif
-
 // our image control
 
 BEGIN_EVENT_TABLE(MaskImageCtrl, wxScrolledWindow)
@@ -118,11 +111,35 @@ void MaskImageCtrl::SetMaskMode(bool newMaskMode)
 void MaskImageCtrl::setImage(const std::string & file, HuginBase::MaskPolygonVector newMask, HuginBase::MaskPolygonVector masksToDraw, ImageRotation rot)
 {
     DEBUG_TRACE("setting Image " << file);
-    m_imageFilename = file;
-    wxString fn(m_imageFilename.c_str(),HUGIN_CONV_FILENAME);
-    if (wxFileName::FileExists(fn))
+    if(!file.empty())
     {
-        m_img = ImageCache::getInstance().getImage(m_imageFilename);
+        try
+        {
+            m_img = ImageCache::getInstance().getImage(file);
+        }
+        catch (...)
+        {
+            // loading of image failed, set all to empty values
+            m_imageFilename = "";
+            m_maskEditState = NO_IMAGE;
+            m_bitmap = wxBitmap();
+            // delete the image (release shared_ptr)
+            // create an empty image.
+            m_img = ImageCache::EntryPtr(new ImageCache::Entry);
+            HuginBase::MaskPolygonVector mask;
+            m_imageMask = mask;
+            m_masksToDraw = mask;
+            m_imgRotation = ROT0;
+            setActiveMask(UINT_MAX, false);
+            SetVirtualSize(100, 100);
+            Refresh(true);
+            // now notify main frame to remove the image from project
+            wxCommandEvent e(EVT_LOADING_FAILED);
+            e.SetString(wxString(file.c_str(), HUGIN_CONV_FILENAME));
+            MainFrame::Get()->GetEventHandler()->AddPendingEvent(e);
+            return;
+        }
+        m_imageFilename = file;
         if(m_maskMode)
         {
             m_maskEditState = NO_MASK;
@@ -1135,7 +1152,7 @@ void MaskImageCtrl::DrawCrop(wxDC & dc)
 
 void MaskImageCtrl::OnDraw(wxDC & dc)
 {
-    if(m_maskEditState!=NO_IMAGE)
+    if(m_maskEditState!=NO_IMAGE && m_bitmap.IsOk())
     {
         int offset=scale(HuginBase::maskOffset);
         //draw border around image to allow drawing mask over boudaries of image
@@ -1348,27 +1365,7 @@ void MaskImageCtrl::rescaleImage()
     m_bitmap=wxBitmap(img);
 
     //create disabled m_bitmap for drawing active masks
-#if wxCHECK_VERSION(2,9,0)
     img = img.ConvertToDisabled(192);
-#else
-    {
-        int width = img.GetWidth();
-        int height = img.GetHeight();
-        for (int y = height-1; y >= 0; --y)
-        {
-            for (int x = width-1; x >= 0; --x)
-            {
-                unsigned char* data = img.GetData() + (y*(width*3))+(x*3);
-                unsigned char* r = data;
-                unsigned char* g = data+1;
-                unsigned char* b = data+2;
-                *r=(unsigned char)wxMin(0.6*(*r)+77,255);
-                *b=(unsigned char)wxMin(0.6*(*b)+77,255);
-                *g=(unsigned char)wxMin(0.6*(*g)+77,255);
-            }
-        }
-    }
-#endif
     m_disabledBitmap=wxBitmap(img);
     if (m_imgRotation == ROT90 || m_imgRotation == ROT270)
     {
@@ -1378,7 +1375,7 @@ void MaskImageCtrl::rescaleImage()
     {
         SetVirtualSize(m_imageSize.GetWidth(), m_imageSize.GetHeight());
     };
-    SetScrollRate(1,1);
+    SetScrollRate(std::max<int>(1, m_bitmap.GetWidth() / 75), std::max<int>(1, m_bitmap.GetHeight() / 75));
     Refresh(true);
 };
 
