@@ -505,19 +505,14 @@ wxString LoadLensOperation::GetLabel()
 PanoCommand::PanoCommand* LoadLensOperation::GetInternalCommand(wxWindow* parent, HuginBase::Panorama& pano, HuginBase::UIntSet images)
 {
     HuginBase::StandardImageVariableGroups variable_groups(pano);
-    if(images.size()==1)
+    HuginBase::UIntSet lensImages = variable_groups.getLenses().getPartsSet()[variable_groups.getLenses().getPartNumber(*images.begin())];
+    if (!m_fromDatabase && images.size() == 1 && lensImages.size() > 1)
     {
+        // database is always linking the parameters, so no need to ask user
         if(wxMessageBox(_("You selected only one image.\nShould the loaded parameters be applied to all images with the same lens?"),_("Question"), wxICON_QUESTION | wxYES_NO)==wxYES)
         {
-            unsigned int lensNr = variable_groups.getLenses().getPartNumber(*images.begin());
             // get all images with the current lens.
-            for (size_t i = 0; i < pano.getNrOfImages(); i++)
-            {
-                if (variable_groups.getLenses().getPartNumber(i) == lensNr)
-                {
-                    images.insert(i);
-                };
-            };
+            std::copy(lensImages.begin(), lensImages.end(), std::inserter(images, images.end()));
         };
     };
     vigra::Size2D sizeImg0=pano.getImage(*(images.begin())).getSize();
@@ -811,9 +806,10 @@ ResetOperation::ResetOperation(ResetMode newResetMode)
     m_resetExposure=0;
     if(m_resetMode==RESET_PHOTOMETRICS)
     {
-        m_resetExposure=1;
+        m_resetExposure=3;
     };
     m_resetVignetting=(m_resetMode==RESET_PHOTOMETRICS);
+    m_resetColor = 0;
     if(m_resetMode==RESET_PHOTOMETRICS)
     {
         m_resetColor=1;
@@ -935,7 +931,7 @@ PanoCommand::PanoCommand* ResetOperation::GetInternalCommand(wxWindow* parent, H
         };
         if(m_resetExposure>0)
         {
-            if(m_resetExposure==1)
+            if(m_resetExposure==1 || m_resetExposure==3)
             {
                 if (pano.getImage(imgNr).ExposureValueisLinked())
                 {
@@ -945,7 +941,7 @@ PanoCommand::PanoCommand* ResetOperation::GetInternalCommand(wxWindow* parent, H
                 }
                 //reset to exif value
                 double eV=srcImg.calcExifExposureValue();
-                if(eV!=0)
+                if ((m_resetExposure == 1 && eV != 0) || m_resetExposure == 3)
                 {
                     map_get(ImgVars,"Eev").setValue(eV);
                 }
@@ -1080,10 +1076,6 @@ bool ResetOperation::ShowDialog(wxWindow* parent)
     bool checkPhotometric;
     switch(m_resetMode)
     {
-        case RESET_DIALOG:
-            checkGeometric=true;
-            checkPhotometric=true;
-            break;
         case RESET_DIALOG_LENS:
             reset_dlg.LimitToGeometric();
             checkGeometric=true;
@@ -1093,6 +1085,12 @@ bool ResetOperation::ShowDialog(wxWindow* parent)
             reset_dlg.LimitToPhotometric();
             checkGeometric=false;
             checkPhotometric=true;
+            break;
+        case RESET_DIALOG:
+        default:
+            checkGeometric=true;
+            checkPhotometric=true;
+            break;
     };
     if(reset_dlg.ShowModal()==wxID_OK)
     {
