@@ -19,19 +19,23 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "Batch.h"
 #include <wx/stdpaths.h>
+#ifdef __WXMSW__
+#include <powrprof.h>
+#pragma comment(lib, "PowrProf.lib")
+#endif
 
 BEGIN_EVENT_TABLE(Batch, wxFrame)
     EVT_END_PROCESS(-1, Batch::OnProcessTerminate)
 END_EVENT_TABLE()
 
-#if _WINDOWS && defined Hugin_shared
+#if defined _WIN32 && defined Hugin_shared
 DEFINE_LOCAL_EVENT_TYPE(EVT_BATCH_FAILED)
 DEFINE_LOCAL_EVENT_TYPE(EVT_INFORMATION)
 DEFINE_LOCAL_EVENT_TYPE(EVT_UPDATE_PARENT)
@@ -41,17 +45,19 @@ DEFINE_EVENT_TYPE(EVT_INFORMATION)
 DEFINE_EVENT_TYPE(EVT_UPDATE_PARENT)
 #endif
 
-Batch::Batch(wxFrame* parent, wxString path, bool bgui) : wxFrame(parent,wxID_ANY,_T("Batch"))
+Batch::Batch(wxFrame* parent) : wxFrame(parent, wxID_ANY, _T("Batch"))
 {
     //default flag settings
-    parallel = false;
     deleteFiles = false;
-    shutdown = false;
+    atEnd = DO_NOTHING;
+#if wxCHECK_VERSION(3,1,0)
+    m_resBlocker = NULL;
+#endif
     overwrite = true;
     verbose = false;
     autoremove = false;
     autostitch = false;
-    gui = bgui;
+    saveLog = false;
     m_cancelled = false;
     m_paused = false;
     m_running = false;
@@ -61,22 +67,16 @@ Batch::Batch(wxFrame* parent, wxString path, bool bgui) : wxFrame(parent,wxID_AN
     // Required to access the preferences of hugin
     //SetAppName(wxT("hugin"));
 
-    // setup the environment for the different operating systems
-    wxConfigBase* config=wxConfigBase::Get();
-#if defined __WXMSW__
-    wxString huginExeDir = getExePath(path);
+}
 
-    wxString huginRoot;
-    wxFileName::SplitPath(huginExeDir, &huginRoot, NULL, NULL);
-
-    progs = getPTProgramsConfig(huginExeDir, config);
-    progsAss = getAssistantProgramsConfig(huginExeDir, config);
-#else
-    // add the locale directory specified during configure
-    progs = getPTProgramsConfig(wxT(""), config);
-    progsAss = getAssistantProgramsConfig(wxT(""), config);
+Batch::~Batch()
+{
+#if wxCHECK_VERSION(3,1,0)
+    if (m_resBlocker != NULL)
+    {
+        delete m_resBlocker;
+    };
 #endif
-
 }
 
 void Batch::AddAppToBatch(wxString app)
@@ -100,7 +100,7 @@ void Batch::AddProjectToBatch(wxString projectFile, wxString outputFile,Project:
     else
     {
         //on output set as "", it defaults to same path and name as project file
-        Project* proj = new Project(projectName.GetFullPath(),projectName.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR) + projectName.GetName());
+        Project* proj = new Project(projectName.GetFullPath(),wxT(""));
         m_projList.Add(proj);
     }
 }
@@ -128,38 +128,83 @@ void Batch::AppendBatchFile(wxString file)
         aFile.GetTimes(NULL,NULL,&m_lastmod);
         wxFileInputStream fileStream(file);
         wxString projectName = _T("");
+#ifdef __WXMSW__
+        wxTextInputStream textStream(fileStream, wxT(" \t"), wxConvLocal);
+#else
         wxTextInputStream textStream(fileStream);
+#endif
 
         //TO-DO: batch file error checking?
         //first line in file is idGenerator, we save it a temp variable, cause it gets set when adding projects
         long idGenTemp = 1;
+        if (fileStream.Eof())
+        {
+            return;
+        };
         textStream.ReadLine().ToLong(&idGenTemp);
         //then for each project: project path, prefix, id, status, skip
+        if (fileStream.Eof())
+        {
+            return;
+        };
         while((projectName = textStream.ReadLine()).Cmp(wxT(""))!=0)
         {
-            //we add project to internal list
-            wxString line=textStream.ReadLine();
-            if(line.IsEmpty())
+            // read all line, check before reading
+            if (fileStream.Eof())
             {
-                AddProjectToBatch(projectName,wxT(""),Project::DETECTING);
+                break;
+            };
+            wxString line=textStream.ReadLine();
+            if (fileStream.Eof())
+            {
+                break;
+            };
+            long id;
+            textStream.ReadLine().ToLong(&id);
+            long status;
+            if (fileStream.Eof())
+            {
+                break;
+            };
+            textStream.ReadLine().ToLong(&status);
+            if (fileStream.Eof())
+            {
+                break;
+            };
+            const bool skip = textStream.ReadLine().StartsWith(_T("T"));
+
+            //we add project to internal list
+            if (id == -1)
+            {
+                AddAppToBatch(projectName);
             }
             else
             {
-                AddProjectToBatch(projectName,line);
+                if (line.IsEmpty())
+                {
+                    AddProjectToBatch(projectName, wxT(""), Project::DETECTING);
+                }
+                else
+                {
+                    AddProjectToBatch(projectName, line);
+                };
             };
-            textStream.ReadLine().ToLong(&m_projList.Last().id);
-            long status;
-            textStream.ReadLine().ToLong(&status);
             //if status was RUNNING or PAUSED, we set it to FAILED
             if(status==(long)Project::RUNNING || status==(long)Project::PAUSED)
             {
                 status=(long)Project::FAILED;
             }
+            m_projList.Last().id = id;
             m_projList.Last().status = (Project::Status)status;
-            if(textStream.ReadLine().StartsWith(_T("T")))
+            if(skip)
             {
                 m_projList.Last().skip = true;
-            }
+            };
+
+            if (fileStream.Eof())
+            {
+                break;
+            };
         }
         //we set the id generator we got from file
         Project::idGenerator = idGenTemp;
@@ -173,6 +218,13 @@ void Batch::CancelBatch()
     {
         CancelProject(i);
     }
+#if wxCHECK_VERSION(3,1,0)
+    if (m_resBlocker != NULL)
+    {
+        delete m_resBlocker;
+        m_resBlocker = NULL;
+    };
+#endif
     m_running = false;
 }
 void Batch::CancelProject(int index)
@@ -193,104 +245,27 @@ void Batch::ChangePrefix(int index, wxString newPrefix)
     m_projList.Item(index).prefix = newPrefix;
 }
 
-bool Batch::CheckProjectExistence() //used only in console version
-{
-#ifdef __WXMSW__  //on windows we run a loop every second to check if running processes are still active
-    bool exist = true;
-    HANDLE process;
-    DWORD exitState;
-    while(exist)
-    {
-        exist = false;
-
-        for(unsigned int i=0; i<m_stitchFrames.GetCount(); i++)
-        {
-            try
-            {
-                process = OpenProcess(PROCESS_QUERY_INFORMATION,true,m_stitchFrames.Item(i)->GetProcessId());
-                GetExitCodeProcess(process,&exitState);
-            }
-            catch(::exception e)
-            {
-                exitState=1;
-            }
-            if(exitState==STILL_ACTIVE)
-            {
-                exist=true;
-            }
-            else if(exitState!=0)
-            {
-                SetStatus(GetIndex(m_stitchFrames.Item(i)->GetProjectId()),Project::FAILED);
-            }
-            CloseHandle(process);
-        }
-        wxSleep(1);
-    }
-#else //not __WXMSW__, on Linux we wait for each of the processes to complete
-    int status;
-    int pid;
-    for(unsigned int i=0; i<m_stitchFrames.GetCount(); i++)
-    {
-        pid = m_stitchFrames.Item(i)->GetProcessId();
-        if(waitpid(pid,&status,0)==-1)
-        {
-            SetStatus(GetIndex(m_stitchFrames.Item(i)->GetProjectId()),Project::FAILED);    //we set to failed if waitpid terminated with an error
-        }
-        if(!WIFEXITED(status) || WEXITSTATUS(status)!=0)
-        {
-            SetStatus(GetIndex(m_stitchFrames.Item(i)->GetProjectId()),Project::FAILED);    //we set to failed if child terminated abnormally or with a bad exit code
-        }
-    }
-#endif
-
-    wxProcessEvent event;
-    for(int i=m_stitchFrames.GetCount()-1; i>=0; i--)
-    {
-        event.SetId(m_stitchFrames.Item(i)->GetProjectId());
-        if(GetStatus(GetIndex(m_stitchFrames.Item(i)->GetProjectId()))==Project::FAILED)
-        {
-            event.SetTimestamp(-1);    //a failed exit code cannot be set to a wxWidgets event, so we fake it inside event's timestamp
-        }
-        OnProcessTerminate(event);
-    }
-    return true;
-}
-
 int Batch::ClearBatch()
 {
     if(m_stitchFrames.GetCount()!=0)
     {
-        if(gui)
-        {
-            wxMessageDialog message(this, _("Cannot clear batch in progress.\nDo you want to cancel it?"),
-#ifdef _WINDOWS
-                                    _("PTBatcherGUI"),
+        wxMessageDialog message(this, _("Cannot clear batch in progress.\nDo you want to cancel it?"),
+#ifdef _WIN32
+                                _("PTBatcherGUI"),
 #else
-                                    wxT(""),
+                                wxT(""),
 #endif
-                                    wxYES | wxCANCEL | wxICON_INFORMATION);
-            if(message.ShowModal()==wxID_YES)
-            {
-                CancelBatch();
-
-                //we set a flag so we don't process terminating events
-                m_clearedInProgress = true;
-                Project::idGenerator=1;
-                m_projList.Clear();
-                if(gui)
-                {
-                    ((wxFrame*)GetParent())->SetStatusText(_("Cleared batch."));
-                }
-                else if(verbose)
-                {
-                    cout << "Cleared batch." << endl;
-                }
-                return 2;
-            }
-        }
-        else if(verbose)
+                                wxYES_NO | wxICON_INFORMATION);
+        if(message.ShowModal()==wxID_YES)
         {
-            cout << "Error: Cannot clear batch in progress." << endl;
+            CancelBatch();
+
+            //we set a flag so we don't process terminating events
+            m_clearedInProgress = true;
+            Project::idGenerator=1;
+            m_projList.Clear();
+            ((wxFrame*)GetParent())->SetStatusText(_("Cleared batch."));
+            return 2;
         }
         return 1;
         //TO-DO: return
@@ -299,14 +274,7 @@ int Batch::ClearBatch()
     {
         Project::idGenerator=1;
         m_projList.Clear();
-        if(gui)
-        {
-            ((wxFrame*)GetParent())->SetStatusText(_("Cleared batch."));
-        }
-        else if(verbose)
-        {
-            cout << "Cleared batch." << endl;
-        }
+        ((wxFrame*)GetParent())->SetStatusText(_("Cleared batch."));
         return 0;
     }
 }
@@ -387,14 +355,10 @@ Project::Status Batch::GetStatus(int index)
     {
         return m_projList.Item(index).status;
     }
-    else if(gui)
+    else
     {
         wxMessageBox(wxString::Format(_("Error: Could not get status, project with index %d is not in list."),index),_("Error!"),wxOK | wxICON_INFORMATION );
     }
-    else if(verbose)
-    {
-        cout << "Error: Could not get status, project with index " << index << " is not in list." << endl;
-    }
     return Project::MISSING;
 }
 
@@ -412,17 +376,17 @@ void Batch::ListBatch()
 {
     if(m_projList.GetCount() == 0)
     {
-        cout << "Batch is empty." << endl;
+        std::cout << "Batch is empty." << std::endl;
     }
     else
     {
-        cout << "List of projects in batch:" << endl <<
-             "[ID] [project path] [output filename] [status]" << endl <<
-             "-------------------------------------" << endl;
+        std::cout << "List of projects in batch:" << std::endl <<
+             "[ID] [project path] [output filename] [status]" << std::endl <<
+             "-------------------------------------" << std::endl;
         for(unsigned int i=0; i<m_projList.GetCount(); i++)
         {
-            cout << m_projList.Item(i).id << "  "	<< (const char*)m_projList.Item(i).path.char_str()  << "  " << (const char*)m_projList.Item(i).prefix.char_str()
-                 << "  " << (const char*)m_projList.Item(i).GetStatusText().char_str() << endl;
+            std::cout << m_projList.Item(i).id << "  "	<< (const char*)m_projList.Item(i).path.char_str()  << "  " << (const char*)m_projList.Item(i).prefix.char_str()
+                 << "  " << (const char*)m_projList.Item(i).GetStatusText().char_str() << std::endl;
         }
     }
 }
@@ -440,14 +404,10 @@ int Batch::LoadBatchFile(wxString file)
         AppendBatchFile(file);
         return 2;
     }
-    else if(gui)
+    else
     {
         wxMessageBox(_("Error: Could not load batch file."));
-    }
-    else if(verbose)
-    {
-        cout << "Error: Could not load batch file." << endl;
-    }
+    };
     return 1;
 }
 
@@ -486,10 +446,6 @@ int Batch::LoadTemp()
     }
     //we load the data from the temp file
     AppendBatchFile(workingDir->GetName()+wxFileName::GetPathSeparator()+temp);
-    if(verbose && !gui)
-    {
-        cout << "Loaded temp file." << endl;
-    }
     return 0;
 }
 
@@ -532,12 +488,9 @@ void Batch::OnProcessTerminate(wxProcessEvent& event)
             m_paused = false;
         }
         i = GetIndex(event.GetId());
-        if (event.GetExitCode() != 0 || event.GetTimestamp()==-1) //timestamp is used as a fake exit code because it cannot be set manually
+        wxString savedLogfile=wxEmptyString;
+        if(saveLog || event.GetExitCode() != 0 || event.GetTimestamp()==-1)
         {
-            m_projList.Item(i).status=Project::FAILED;
-            struct FailedProject failedProject;
-            failedProject.project=m_projList.Item(i).path;
-            failedProject.logfile=wxEmptyString;
             //get filename for automatic saving of log file
             wxFileName logFile(m_projList.Item(i).path);
             logFile.MakeAbsolute();
@@ -552,11 +505,18 @@ void Batch::OnProcessTerminate(wxProcessEvent& event)
             if(i<1000)
             {
                 //now save log file
-                if(((RunStitchFrame*)(event.GetEventObject()))->SaveLog(logFile.GetFullPath()))
+                if((static_cast<RunStitchFrame*>(event.GetEventObject()))->SaveLog(logFile.GetFullPath()))
                 {
-                    failedProject.logfile=logFile.GetFullPath();
+                    savedLogfile=logFile.GetFullPath();
                 }
             };
+        };
+        if (event.GetExitCode() != 0 || event.GetTimestamp()==-1) //timestamp is used as a fake exit code because it cannot be set manually
+        {
+            m_projList.Item(i).status=Project::FAILED;
+            struct FailedProject failedProject;
+            failedProject.project=m_projList.Item(i).path;
+            failedProject.logfile=savedLogfile;
             //remember failed project
             m_failedProjects.push_back(failedProject);
         }
@@ -564,7 +524,7 @@ void Batch::OnProcessTerminate(wxProcessEvent& event)
         {
             m_projList.Item(i).status=Project::FINISHED;
             // don't sent event for command app
-            if(gui && m_projList.Item(i).id>=0)
+            if(m_projList.Item(i).id>=0)
             {
                 bool notifyParent=false;
                 if(autostitch && m_projList.Item(i).target==Project::DETECTING)
@@ -576,6 +536,7 @@ void Batch::OnProcessTerminate(wxProcessEvent& event)
                 if(autoremove)
                 {
                     RemoveProjectAtIndex(i);
+                    SaveTemp();
                     notifyParent=true;
                 };
                 if(notifyParent)
@@ -589,109 +550,100 @@ void Batch::OnProcessTerminate(wxProcessEvent& event)
         {
             if(AllDone())
             {
+                SaveTemp();
                 m_running = false;
+#if wxCHECK_VERSION(3,1,0)
+                if (m_resBlocker != NULL)
+                {
+                    delete m_resBlocker;
+                    m_resBlocker = NULL;
+                };
+#endif
                 if(NoErrors())
                 {
-                    if(gui)
-                    {
-                        wxCommandEvent e(EVT_INFORMATION,wxID_ANY);
-                        e.SetString(_("Batch successfully completed."));
-                        GetParent()->GetEventHandler()->AddPendingEvent(e);
-                    }
-                    else
-                        //cout << "Project \"" << m_projList.Item(i).path.char_str() << "\" finished. Batch successfully completed." << endl;
-                    {
-                        cout << "Batch successfully completed." << endl;
-                    }
+                    wxCommandEvent e(EVT_INFORMATION,wxID_ANY);
+                    e.SetString(_("Batch successfully completed."));
+                    // setting int to 1 to indicate we are finished
+                    e.SetInt(1);
+                    GetParent()->GetEventHandler()->AddPendingEvent(e);
                 }
                 else
                 {
-                    if(gui)
+                    ((wxFrame*)GetParent())->SetStatusText(_("Batch completed with errors."));
+                    if(atEnd==DO_NOTHING)
                     {
-                        SetStatusText(_("Batch completed with errors."));
-                        if(!shutdown)
-                        {
-                            if(gui)
-                            {
-                                //notify parent, that at least one project failed
-                                wxCommandEvent e(EVT_BATCH_FAILED,wxID_ANY);
-                                GetParent()->GetEventHandler()->AddPendingEvent(e);
-                            };
-                        };
-                    }
-                    else
-                        //cout << "Project \"" << m_projList.Item(i).path.char_str() << "\" finished. Batch completed with errors." << endl;
-                    {
-                        cout << "Batch completed with errors." << endl;
-                    }
-                }
-                if(shutdown)	//after we are finished we turn off the computer if checked
+                        //notify parent, that at least one project failed
+                        // show dialog only if we don't shutdown the computer or end PTBatcherGUI
+                        wxCommandEvent e(EVT_BATCH_FAILED,wxID_ANY);
+                        GetParent()->GetEventHandler()->AddPendingEvent(e);
+                    };
+                };
+                switch (atEnd)
                 {
-                    if(gui)
-                    {
-                        wxProgressDialog progress(_("Initializing shutdown..."), _("Shutting down..."),49,this,
-                                                  wxPD_AUTO_HIDE | wxPD_SMOOTH | wxPD_APP_MODAL | wxPD_CAN_ABORT | wxPD_CAN_SKIP);
-                        progress.Fit();
-                        int i = 0;
-                        bool skip = false;
-                        while(progress.Update(i, _("Shutting down..."),&skip))
+                    case DO_NOTHING:
+                        // no action needed
+                        break;
+                    case CLOSE_PTBATCHERGUI:
+                        GetParent()->Close();
+                        break;
+                    case SHUTDOWN:
                         {
-                            if(skip || i==50)
+                            wxProgressDialog progress(_("Initializing shutdown..."), _("Shutting down..."), 49, this,
+                                wxPD_AUTO_HIDE | wxPD_SMOOTH | wxPD_APP_MODAL | wxPD_CAN_ABORT | wxPD_CAN_SKIP);
+                            progress.Fit();
+                            int i = 0;
+                            bool skip = false;
+                            while (progress.Update(i, _("Shutting down..."), &skip))
                             {
-                                /*wxMessageDialog message(this,_T("Krneksa"));
-                                message.ShowModal();
-                                break;*/
-                                wxShutdown(wxSHUTDOWN_POWEROFF);
-                            }
-                            i++;
+                                if (skip || i == 50)
+                                {
+                                    wxShutdown(wxSHUTDOWN_POWEROFF);
+                                }
+                                i++;
 #if defined __WXMSW__
-                            Sleep(200);
+                                Sleep(200);
 #else
-                            sleep(200);
+                                sleep(200);
 #endif
+                            }
+                            progress.Close();
                         }
-                        progress.Close();
-                    }
-                    else
-                    {
-                        if(!wxShutdown(wxSHUTDOWN_POWEROFF))
+                        break;
+                    case SUSPEND:
+                    case HIBERNATE:
+#ifdef __WXMSW__
                         {
-                            if(gui)
-#ifdef __WXMSW__
-                                wxMessageBox(_("Error shutting down."));
-#else
-                                wxMessageBox(_("Error shutting down. Do you have root privileges?"));
+                            wxString progressCaption(_("Prepare to hibernate..."));
+                            wxString progressLabel(_("Initializing hibernating..."));
+                            if (atEnd == SUSPEND)
+                            {
+                                progressCaption = wxString(_("Prepare to suspend..."));
+                                progressLabel = wxString(_("Initializing suspend mode..."));
+                            };
+                            wxProgressDialog progress(progressLabel, progressCaption, 49, this,
+                                wxPD_AUTO_HIDE | wxPD_SMOOTH | wxPD_APP_MODAL | wxPD_CAN_ABORT | wxPD_CAN_SKIP);
+                            progress.Fit();
+                            int i = 0;
+                            bool skip = false;
+                            while (progress.Update(i, progressCaption, &skip))
+                            {
+                                if (skip || i == 50)
+                                {
+                                    SetSuspendState(atEnd==HIBERNATE, false, false);
+                                    break;
+                                }
+                                i++;
+                                Sleep(200);
+                            }
+                            progress.Close();
+                        };
 #endif
-                            else
-#ifdef __WXMSW__
-                                cout << "Error shutting down." << endl;
-#else
-                                cout << "Error shutting down. Do you have root privileges?" << endl;
-#endif
-                        }
-                    }
-                }
+                        break;
+                };
             }
             else
             {
-                if(parallel)	//if we are running in parallel
-                {
-                    //the last executed process in parallel runs next
-                    if(GetRunningCount() == 0)
-                    {
-                        //SetStatusText(_T("Project \""+m_projList.Item(i).path)+_T("\" finished. Running next project..."));
-                        RunNextInBatch();
-                    }
-                    else
-                    {
-                        //SetStatusText(_T("Project \""+m_projList.Item(i).path)+_T("\" finished. Waiting for all in parallel to complete..."));
-                    }
-                }
-                else
-                {
-                    //SetStatusText(_T("Project \""+m_projList.Item(i).path)+_T("\" finished. Running next project..."));
-                    RunNextInBatch();
-                }
+                RunNextInBatch();
             }
         }
         else
@@ -708,19 +660,23 @@ void Batch::OnProcessTerminate(wxProcessEvent& event)
 
 bool Batch::OnStitch(wxString scriptFile, wxString outname, int id)
 {
+    // delete the existing wxConfig to force reloading of settings from file/registy
+    delete wxConfigBase::Set((wxConfigBase*)NULL);
+    wxConfigBase* config = wxConfigBase::Get();
     if(wxIsEmpty(scriptFile))
     {
-        wxString defaultdir = wxConfigBase::Get()->Read(wxT("/actualPath"),wxT(""));
+        wxString defaultdir = config->Read(wxT("/actualPath"),wxT(""));
         wxFileDialog dlg(0,
-                         _("Specify project source project file"),
+                         _("Specify project file"),
                          defaultdir, wxT(""),
-                         _("Project files (*.pto,*.ptp,*.pts,*.oto)|*.pto;*.ptp;*.pts;*.oto;|All files (*)|*"),
+                         _("Project files (*.pto)|*.pto|All files (*)|*"),
                          wxFD_OPEN, wxDefaultPosition);
 
         dlg.SetDirectory(wxConfigBase::Get()->Read(wxT("/actualPath"),wxT("")));
         if (dlg.ShowModal() == wxID_OK)
         {
-            wxConfig::Get()->Write(wxT("/actualPath"), dlg.GetDirectory());  // remember for later
+            config->Write(wxT("/actualPath"), dlg.GetDirectory());  // remember for later
+            config->Flush();
             wxFileDialog dlg2(0,_("Specify output prefix"),
                               wxConfigBase::Get()->Read(wxT("/actualPath"),wxT("")),
                               wxT(""), wxT(""),
@@ -739,16 +695,11 @@ bool Batch::OnStitch(wxString scriptFile, wxString outname, int id)
         }
         else     // bail
         {
+            wxLogError(_("No project files specified"));
             return false;
-            wxLogError( _("No project files specified"));
         }
     }
 
-    if(!gui)
-    {
-        cout << "Stitching with input file " << (const char*)scriptFile.mb_str(wxConvLocal) << "..." << endl;
-    }
-
     // check output filename
     wxFileName outfn(outname);
     wxString ext = outfn.GetExt();
@@ -764,7 +715,7 @@ bool Batch::OnStitch(wxString scriptFile, wxString outname, int id)
 
     RunStitchFrame* stitchFrame = new RunStitchFrame(this, wxT("Hugin Stitcher"), wxDefaultPosition, wxSize(640,600));
     stitchFrame->SetProjectId(id);
-    if(verbose && gui)
+    if(verbose)
     {
         stitchFrame->Show( true );
         wxTheApp->SetTopWindow( stitchFrame );
@@ -777,7 +728,7 @@ bool Batch::OnStitch(wxString scriptFile, wxString outname, int id)
         stitchFrame->m_stitchPanel->SetOverwrite(true);
     }
 
-    bool n = stitchFrame->StitchProject(scriptFile, outname, progs);
+    bool n = stitchFrame->StitchProject(scriptFile, outname);
     if(n)
     {
         m_stitchFrames.Add(stitchFrame);
@@ -792,14 +743,11 @@ bool Batch::OnStitch(wxString scriptFile, wxString outname, int id)
 
 bool Batch::OnDetect(wxString scriptFile, int id)
 {
-    if(!gui)
-    {
-        cout << "Running assistant with input file " << (const char*)scriptFile.mb_str(wxConvLocal) << "..." << endl;
-    }
-
-    RunStitchFrame* stitchFrame = new RunStitchFrame(this, wxT("Hugin Assistant"), wxDefaultPosition, wxSize(640,600));
+    // delete the existing wxConfig to force reloading of settings from file/registy
+    delete wxConfigBase::Set((wxConfigBase*)NULL);
+    RunStitchFrame* stitchFrame = new RunStitchFrame(this, wxT("Hugin Assistant"), wxDefaultPosition, wxSize(640, 600));
     stitchFrame->SetProjectId(id);
-    if(verbose && gui)
+    if(verbose)
     {
         stitchFrame->Show( true );
         wxTheApp->SetTopWindow( stitchFrame );
@@ -808,7 +756,7 @@ bool Batch::OnDetect(wxString scriptFile, int id)
     wxFileName basename(scriptFile);
     stitchFrame->SetTitle(wxString::Format(_("%s - Assistant"), basename.GetName().c_str()));
 
-    bool n = stitchFrame->DetectProject(scriptFile, progsAss);
+    bool n = stitchFrame->DetectProject(scriptFile);
     if(n)
     {
         m_stitchFrames.Add(stitchFrame);
@@ -857,19 +805,14 @@ void Batch::PauseBatch()
 
 void Batch::RemoveProject(int id)
 {
-    int index;
-    if((index=GetIndex(id)) != -1)
+    if(GetIndex(id) != -1)
     {
         RemoveProjectAtIndex(GetIndex(id));
     }
-    else if(gui)
+    else
     {
         wxMessageBox(wxString::Format(_("Error removing, project with id %d is not in list."),id),_("Error!"),wxOK | wxICON_INFORMATION );
     }
-    else if(verbose)
-    {
-        cout << "Error: Project with id " << id << " is not in list." << endl;
-    }
 }
 
 void Batch::RemoveProjectAtIndex(int selIndex)
@@ -884,14 +827,7 @@ void Batch::RemoveProjectAtIndex(int selIndex)
         {
             if(!wxRemoveFile(file.GetFullPath()))
             {
-                if(gui)
-                {
-                    wxMessageBox( _("Error: Could not delete project file ")+file.GetFullPath(),_("Error!"),wxOK | wxICON_INFORMATION );
-                }
-                else if(verbose)
-                {
-                    cout << "Error: Could not delete project file " << (const char*)file.GetFullPath().char_str() << endl;
-                }
+                wxMessageBox( _("Error: Could not delete project file ")+file.GetFullPath(),_("Error!"),wxOK | wxICON_INFORMATION );
             }
         }
     }
@@ -904,25 +840,17 @@ void Batch::RemoveProjectAtIndex(int selIndex)
 
 void Batch::RunBatch()
 {
-    if(!gui && m_projList.GetCount() == 0)
-    {
-        cout << "Batch is empty." << endl;
-    }
-    else if(!m_running)
+    if(!m_running)
     {
         m_failedProjects.clear();
-        if(gui)
-        {
-            ((wxFrame*)GetParent())->SetStatusText(_("Running batch..."));
-        }
-        else
-        {
-            cout << "Running batch..." << endl;
-        }
+        ((wxFrame*)GetParent())->SetStatusText(_("Running batch..."));
         m_running = true;
+#if wxCHECK_VERSION(3,1,0)
+        m_resBlocker = new wxPowerResourceBlocker(wxPOWER_RESOURCE_SYSTEM, _("PTBatcherGUI is stitching"));
+#endif
         RunNextInBatch();
     }
-    else if(gui)
+    else
     {
         ((wxFrame*)GetParent())->SetStatusText(_("Batch already in progress."));
     }
@@ -938,113 +866,43 @@ void Batch::RunNextInBatch()
         //execute command line instructions
         if(m_projList.Item(i).id<0)
         {
-            if(gui)
-            {
-                SetStatusText(_("Running command \"")+m_projList.Item(i).path+_T("\""));
-            }
-            else
-            {
-                cout << "Running command \"" << (const char*)m_projList.Item(i).path.char_str() << "\"" << endl;
-            }
+            SetStatusText(wxString::Format(_("Running command \"%s\""), m_projList.Item(i).path.c_str()));
             m_projList.Item(i).status=Project::RUNNING;
             //we create a fake stitchFrame, so program waits for app to complete
-            if(!gui)
+            if(wxExecute(m_projList.Item(i).path, wxEXEC_SYNC)==0)
             {
-                RunStitchFrame* stitchFrame = new RunStitchFrame(this, wxT("Hugin Stitcher"), wxDefaultPosition, wxSize(640,600));
-                stitchFrame->SetProjectId(m_projList.Item(i).id);
-
-                repeat = false;
-                int pid = wxExecute(m_projList.Item(i).path, wxEXEC_ASYNC);
-                stitchFrame->SetProcessId(pid);
-                m_stitchFrames.Add(stitchFrame);
+                m_projList.Item(i).status=Project::FINISHED;
             }
             else
             {
-                if(wxExecute(m_projList.Item(i).path, wxEXEC_SYNC)==0)
-                {
-                    m_projList.Item(i).status=Project::FINISHED;
-                }
-                else
-                {
-                    m_projList.Item(i).status=Project::FAILED;
-                }
+                m_projList.Item(i).status=Project::FAILED;
             }
         }
         else
         {
-            //we run in sequence
-            if(!parallel)
+            m_projList.Item(i).status=Project::RUNNING;
+            m_running = true;
+            if(m_projList.Item(i).target==Project::STITCHING)
             {
-                m_projList.Item(i).status=Project::RUNNING;
-                m_running = true;
-                if(m_projList.Item(i).target==Project::STITCHING)
-                {
-                    if(gui)
-                    {
-                        wxCommandEvent e(EVT_INFORMATION,wxID_ANY);
-                        e.SetString(wxString::Format(_("Now stitching: %s"),m_projList.Item(i).path.c_str()));
-                        GetParent()->GetEventHandler()->AddPendingEvent(e);
-                    };
-                    value = OnStitch(m_projList.Item(i).path, m_projList.Item(i).prefix, m_projList.Item(i).id);
-                }
-                else
-                {
-                    if(gui)
-                    {
-                        wxCommandEvent e(EVT_INFORMATION,wxID_ANY);
-                        e.SetString(wxString::Format(_("Now detecting: %s"),m_projList.Item(i).path.c_str()));
-                        GetParent()->GetEventHandler()->AddPendingEvent(e);
-                    };
-                    value = OnDetect(m_projList.Item(i).path,m_projList.Item(i).id);
-                };
-                if(!value)
-                {
-                    m_projList.Item(i).status=Project::FAILED;
-                }
-                else
-                {
-                    repeat = false;
-                }
+                wxCommandEvent e(EVT_INFORMATION,wxID_ANY);
+                e.SetString(wxString::Format(_("Now stitching: %s"),m_projList.Item(i).path.c_str()));
+                GetParent()->GetEventHandler()->AddPendingEvent(e);
+                value = OnStitch(m_projList.Item(i).path, m_projList.Item(i).prefix, m_projList.Item(i).id);
             }
             else
             {
-                while((i = GetFirstAvailable())!=-1)
-                {
-                    if(m_projList.Item(i).id<0)
-                    {
-                        break;
-                    }
-                    m_projList.Item(i).status=Project::RUNNING;
-                    m_running = true;
-                    if(m_projList.Item(i).target==Project::STITCHING)
-                    {
-                        if(gui)
-                        {
-                            wxCommandEvent e(EVT_INFORMATION,wxID_ANY);
-                            e.SetString(wxString::Format(_("Now stitching: %s"),m_projList.Item(i).path.c_str()));
-                            GetParent()->GetEventHandler()->AddPendingEvent(e);
-                        };
-                        value = OnStitch(m_projList.Item(i).path, m_projList.Item(i).prefix, m_projList.Item(i).id);
-                    }
-                    else
-                    {
-                        if(gui)
-                        {
-                            wxCommandEvent e(EVT_INFORMATION,wxID_ANY);
-                            e.SetString(wxString::Format(_("Now detecting: %s"),m_projList.Item(i).path.c_str()));
-                            GetParent()->GetEventHandler()->AddPendingEvent(e);
-                        };
-                        value = OnDetect(m_projList.Item(i).path,m_projList.Item(i).id);
-                    };
-                    if(!value)
-                    {
-                        m_projList.Item(i).status=Project::FAILED;
-                    }
-                    else
-                    {
-                        repeat = false;
-                    }
-                }
+                wxCommandEvent e(EVT_INFORMATION,wxID_ANY);
+                e.SetString(wxString::Format(_("Now detecting: %s"),m_projList.Item(i).path.c_str()));
+                GetParent()->GetEventHandler()->AddPendingEvent(e);
+                value = OnDetect(m_projList.Item(i).path,m_projList.Item(i).id);
+            };
+            if(!value)
+            {
+                m_projList.Item(i).status=Project::FAILED;
+            }
+            else
+            {
+                repeat = false;
             }
         }
     }
@@ -1052,16 +910,16 @@ void Batch::RunNextInBatch()
     {
         m_running = false;
     }
-    else if(!gui)
-    {
-        CheckProjectExistence();
-    }
 }
 
 void Batch::SaveBatchFile(wxString file)
 {
     wxFileOutputStream fileStream(file);
+#ifdef __WXMSW__
+    wxTextOutputStream textStream(fileStream, wxEOL_NATIVE, wxConvLocal);
+#else
     wxTextOutputStream textStream(fileStream);
+#endif
     //we write current idGenerator to file
     wxString line = _T("");
     line << Project::idGenerator;
@@ -1117,10 +975,6 @@ void Batch::SaveTemp()
         suffix = _T("0");
     }
     SaveBatchFile(wxStandardPaths::Get().GetUserConfigDir()+wxFileName::GetPathSeparator()+_T(".ptbt")+suffix);
-    if(verbose && !gui)
-    {
-        cout << "Saved temp file." << endl;
-    }
     //we remove the previous temp file
     if(oldFile.FileExists())
     {
@@ -1134,14 +988,10 @@ void Batch::SetStatus(int index,Project::Status status)
     {
         m_projList.Item(index).status = status;
     }
-    else if(gui)
+    else
     {
         wxMessageBox(wxString::Format(_("Error: Could not set status, project with index %d is not in list."),index),_("Error!"),wxOK | wxICON_INFORMATION );
     }
-    else if(verbose)
-    {
-        cout << "Error: Could not set status, project with index " << index << " is not in list." << endl;
-    }
 }
 
 void Batch::SwapProject(int index)
@@ -1160,7 +1010,7 @@ void Batch::ShowOutput(bool isVisible)
 
 wxString Batch::GetFailedProjectName(unsigned int i)
 {
-    if(i>=0 && i<m_failedProjects.size())
+    if(i<m_failedProjects.size())
     {
         return m_failedProjects[i].project;
     }
@@ -1172,7 +1022,7 @@ wxString Batch::GetFailedProjectName(unsigned int i)
 
 wxString Batch::GetFailedProjectLog(unsigned int i)
 {
-    if(i>=0 && i<m_failedProjects.size())
+    if(i<m_failedProjects.size())
     {
         return m_failedProjects[i].logfile;
     }
