@@ -133,9 +133,6 @@ BatchFrame::BatchFrame(wxLocale* locale, wxString xrc)
     this->SetLocaleAndXRC(locale,xrc);
     m_cancelled = false;
     m_closeThread = false;
-#ifndef __WXMSW__
-    m_help=0;
-#endif
 
     //load xrc resources
     wxXmlResource::Get()->LoadFrame(this, (wxWindow* )NULL, wxT("batch_frame"));
@@ -167,11 +164,7 @@ BatchFrame::BatchFrame(wxLocale* locale, wxString xrc)
     SetIcon(m_iconNormal);
 
     m_batch = new Batch(this);
-#if wxCHECK_VERSION(2,9,4)
     if(wxGetKeyState(WXK_COMMAND))
-#else
-    if(wxGetKeyState(WXK_CONTROL))
-#endif
     {
 #ifdef __WXMAC__
         wxString text(_("You have pressed the Command key."));
@@ -224,21 +217,13 @@ BatchFrame::BatchFrame(wxLocale* locale, wxString xrc)
     };
 #endif
 
-    //wxThreadHelper::Create is deprecated in wxWidgets 2.9+, but its
-    // replacement, CreateThread, does not exist in 2.8. Pick one
-    // depending on the version to a avoid compiler warning(2.9) or error(2.8).
-#if wxCHECK_VERSION(2, 9, 0)
     this->wxThreadHelper::CreateThread();
-#else
-    this->wxThreadHelper::Create();
-#endif
     this->GetThread()->Run();
     //TO-DO: include a batch or project progress gauge?
     projListBox->Fill(m_batch);
     SetDropTarget(new BatchDropTarget());
 
     m_tray = NULL;
-#if wxCHECK_VERSION(3,0,0)
     if (wxTaskBarIcon::IsAvailable())
     {
         // minimize to tray is by default disabled
@@ -250,7 +235,6 @@ BatchFrame::BatchFrame(wxLocale* locale, wxString xrc)
         GetMenuBar()->Check(XRCID("menu_tray"), minTray);
         UpdateTrayIcon(minTray);
     }
-#endif
 
     UpdateTaskBarProgressBar();
 }
@@ -265,14 +249,7 @@ void* BatchFrame::Entry()
         wxFileName aFile(m_batch->GetLastFile());
         if(!aFile.FileExists())
         {
-#if wxCHECK_VERSION(3,0,0)
-            // for thread safety wxWidgets recommends using wxQueueEvent
-            // but this function is not available for older wxWidgets versions
             wxQueueEvent(this, new wxCommandEvent(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_RELOAD_BATCH));
-#else
-            wxCommandEvent evt(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_RELOAD_BATCH);
-            wxPostEvent(this, evt);
-#endif
         }
         else
         {
@@ -280,21 +257,11 @@ void* BatchFrame::Entry()
             aFile.GetTimes(NULL,NULL,&create);
             if(create.IsLaterThan(m_batch->GetLastFileDate()))
             {
-#if wxCHECK_VERSION(3,0,0)
                 wxQueueEvent(this, new wxCommandEvent(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_RELOAD_BATCH));
-#else
-                wxCommandEvent evt(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_RELOAD_BATCH);
-                wxPostEvent(this, evt);
-#endif
             };
         };
         //update project list box
-#if wxCHECK_VERSION(3,0,0)
         wxQueueEvent(this, new wxCommandEvent(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_UPDATE_LISTBOX));
-#else
-        wxCommandEvent evt(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_UPDATE_LISTBOX);
-        wxPostEvent(this, evt);
-#endif
         GetThread()->Sleep(1000);
     }
     return 0;
@@ -512,7 +479,7 @@ void BatchFrame::AddToList(wxString aFile,Project::Target target)
             s=wxString::Format(_("Add project %s to assistant queue."),aFile.c_str());
             break;
     };
-    SetStatusInformation(s,true);
+    SetStatusInformation(s);
     projListBox->AppendProject(m_batch->GetProject(m_batch->GetProjectCount()-1));
     m_batch->SaveTemp();
 }
@@ -523,7 +490,7 @@ void BatchFrame::OnButtonCancel(wxCommandEvent& event)
     GetToolBar()->ToggleTool(XRCID("tool_pause"),false);
     m_cancelled = true;
     m_batch->CancelBatch();
-    SetStatusInformation(_("Batch stopped"),true);
+    SetStatusInformation(_("Batch stopped"));
     if (m_tray)
     {
         m_tray->SetIcon(m_iconNormal, _("Hugin's Batch processor"));
@@ -532,13 +499,14 @@ void BatchFrame::OnButtonCancel(wxCommandEvent& event)
 
 void BatchFrame::OnButtonChangePrefix(wxCommandEvent& event)
 {
-    int selIndex = projListBox->GetSelectedIndex();
-    if(selIndex != -1)
+    const HuginBase::UIntSet selected(projListBox->GetSelectedProjects());
+    if(selected.size()== 1)
     {
-        if(projListBox->GetSelectedProjectTarget()==Project::STITCHING)
+        Project* project = m_batch->GetProject(*selected.begin());
+        if (project->target == Project::STITCHING)
         {
-            wxFileName prefix(projListBox->GetSelectedProjectPrefix());
-            wxFileDialog dlg(0,_("Specify output prefix for project ")+projListBox->GetSelectedProject(),
+            wxFileName prefix(project->prefix);
+            wxFileDialog dlg(0,_("Specify output prefix for project ")+project->path,
                              prefix.GetPath(),
                              prefix.GetFullName(), wxT(""),
                              wxFD_SAVE, wxDefaultPosition);
@@ -572,7 +540,7 @@ void BatchFrame::OnButtonChangePrefix(wxCommandEvent& event)
                 };
 
                 wxString outname(dlg.GetPath());
-                ChangePrefix(selIndex,outname);
+                ChangePrefix(*selected.begin(), outname);
                 //SetStatusText(_T("Changed prefix for "+projListBox->GetSelectedProject()));
                 m_batch->SaveTemp();
             }
@@ -585,8 +553,16 @@ void BatchFrame::OnButtonChangePrefix(wxCommandEvent& event)
     }
     else
     {
-        SetStatusText(_("Please select a project"));
-    }
+        if (selected.empty())
+        {
+            SetStatusText(_("Please select a project"));
+        }
+        else
+        {
+            wxBell();
+            SetStatusText(_("Please select only one project"));
+        };
+    };
 }
 
 void BatchFrame::ChangePrefix(int index,wxString newPrefix)
@@ -626,65 +602,42 @@ void BatchFrame::OnButtonClear(wxCommandEvent& event)
 void BatchFrame::OnButtonHelp(wxCommandEvent& event)
 {
     DEBUG_TRACE("");
-#ifdef __WXMSW__
+#if defined __wxMSW__ && !(wxCHECK_VERSION(3,1,1))
+    // wxWidgets 3.x has a bug, that prevents DisplaySection to work on Win8/10 64 bit
+    // see: http://trac.wxwidgets.org/ticket/14888
+    // so using DisplayContents() and our own implementation of HuginCHMHelpController
     GetHelpController().DisplayHelpPage(wxT("Hugin_Batch_Processor.html"));
 #else
-    if (m_help == 0)
-    {
-#if defined __WXMAC__ && defined MAC_SELF_CONTAINED_BUNDLE
-        // On Mac, xrc/data/help_LOCALE should be in the bundle as LOCALE.lproj/help
-        // which we can rely on the operating sytem to pick the right locale's.
-        wxString strFile = MacGetPathToBundledResourceFile(CFSTR("help"));
-        if(strFile!=wxT(""))
-        {
-            strFile += wxT("/hugin.hhp");
-        }
-        else
-        {
-            wxLogError(wxString::Format(wxT("Could not find help directory in the bundle"), strFile.c_str()));
-            return;
-        }
-#else
-        // find base filename
-        wxString helpFile = wxT("help_") + m_locale->GetCanonicalName() + wxT("/hugin.hhp");
-        DEBUG_INFO("help file candidate: " << helpFile.mb_str(wxConvLocal));
-        //if the language is not default, load custom About file (if exists)
-        wxString strFile = m_xrcPrefix + wxT("data/") + helpFile;
-        if(wxFile::Exists(strFile))
-        {
-            DEBUG_TRACE("Using About: " << strFile.mb_str(wxConvLocal));
-        }
-        else
-        {
-            // try default
-            strFile = m_xrcPrefix + wxT("data/help_en_EN/hugin.hhp");
-        }
-#endif
-
-        if(!wxFile::Exists(strFile))
-        {
-            wxLogError(wxString::Format(wxT("Could not open help file: %s"), strFile.c_str()));
-            return;
-        }
-        DEBUG_INFO(_("help file: ") << strFile.mb_str(wxConvLocal));
-        m_help = new wxHtmlHelpController();
-        m_help->AddBook(strFile);
-    }
-    m_help->Display(wxT("Hugin_Batch_Processor.html"));
-    //DisplayHelp(wxT("Hugin_Batch_Processor.html"));
+    GetHelpController().DisplaySection(wxT("Hugin_Batch_Processor.html"));
 #endif
 }
 
 void BatchFrame::OnButtonMoveDown(wxCommandEvent& event)
 {
-    SwapProject(projListBox->GetSelectedIndex());
-    m_batch->SaveTemp();
+    const HuginBase::UIntSet selected(projListBox->GetSelectedProjects());
+    if (selected.size() == 1)
+    {
+        SwapProject(*selected.begin(), true);
+        m_batch->SaveTemp();
+    }
+    else
+    {
+        wxBell();
+    };
 }
 
 void BatchFrame::OnButtonMoveUp(wxCommandEvent& event)
 {
-    SwapProject(projListBox->GetSelectedIndex()-1);
-    m_batch->SaveTemp();
+    const HuginBase::UIntSet selected(projListBox->GetSelectedProjects());
+    if (selected.size() == 1)
+    {
+        SwapProject(*selected.begin() - 1, false);
+        m_batch->SaveTemp();
+    }
+    else
+    {
+        wxBell();
+    };
 }
 
 void BatchFrame::OnButtonOpenBatch(wxCommandEvent& event)
@@ -714,36 +667,47 @@ void BatchFrame::OnButtonOpenBatch(wxCommandEvent& event)
 void BatchFrame::OnButtonOpenWithHugin(wxCommandEvent& event)
 {
     const wxFileName exePath(wxStandardPaths::Get().GetExecutablePath());
-    if(projListBox->GetSelectedIndex()!=-1)
-        if(projListBox->GetText(projListBox->GetSelectedIndex(),0).Cmp(_T(""))!=0)
+    const HuginBase::UIntSet selected(projListBox->GetSelectedProjects());
+    if (selected.size() == 1)
+    {
+        if (projListBox->GetText(*selected.begin(), 0).Cmp(_T("")) != 0)
 #ifdef __WXMAC__
-            wxExecute(_T("open -b net.sourceforge.hugin.hugin \"" + projListBox->GetSelectedProject()+_T("\"")));
+            wxExecute(_T("open -b net.sourceforge.Hugin \"" + m_batch->GetProject(*selected.begin())->path + _T("\"")));
 #else
-            wxExecute(exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR)+_T("hugin \"" + projListBox->GetSelectedProject()+_T("\" -notips")));
+            wxExecute(exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR) + _T("hugin \"" + m_batch->GetProject(*selected.begin())->path + _T("\" -notips")));
 #endif
         else
         {
             SetStatusText(_("Cannot open app in Hugin."));
-        }
+        };
+    }
     else
     {
-        //ask user if he/she wants to load an empty project
-        wxMessageDialog message(this,_("No project selected. Open Hugin without project?"),
-#ifdef _WIN32
-                                _("PTBatcherGUI"),
-#else
-                                wxT(""),
-#endif
-                                wxYES_NO | wxICON_INFORMATION );
-        if(message.ShowModal() == wxID_YES)
+        if (selected.empty())
         {
-#ifdef __WXMAC__
-            wxExecute(_T("open -b net.sourceforge.hugin.hugin"));
+            //ask user if he/she wants to load an empty project
+            wxMessageDialog message(this, _("No project selected. Open Hugin without project?"),
+#ifdef _WIN32
+                _("PTBatcherGUI"),
 #else
-            wxExecute(exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR)+_T("hugin"));
+                wxT(""),
 #endif
+                wxYES_NO | wxICON_INFORMATION);
+            if (message.ShowModal() == wxID_YES)
+            {
+#ifdef __WXMAC__
+                wxExecute(_T("open -b net.sourceforge.Hugin"));
+#else
+                wxExecute(exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR) + _T("hugin"));
+#endif
+            }
         }
-    }
+        else
+        {
+            wxBell();
+            SetStatusText(_("Please select only one project"));
+        };
+    };
 }
 
 void BatchFrame::OnButtonPause(wxCommandEvent& event)
@@ -754,7 +718,7 @@ void BatchFrame::OnButtonPause(wxCommandEvent& event)
         {
             m_batch->PauseBatch();
             GetToolBar()->ToggleTool(XRCID("tool_pause"),true);
-            SetStatusInformation(_("Batch paused"),true);
+            SetStatusInformation(_("Batch paused"));
             if (m_tray)
             {
                 m_tray->SetIcon(m_iconPaused, _("Pausing processing Hugin's batch queue"));
@@ -764,7 +728,7 @@ void BatchFrame::OnButtonPause(wxCommandEvent& event)
         {
             m_batch->PauseBatch();
             GetToolBar()->ToggleTool(XRCID("tool_pause"),false);
-            SetStatusInformation(_("Continuing batch..."),true);
+            SetStatusInformation(_("Continuing batch..."));
             if (m_tray)
             {
                 m_tray->SetIcon(m_iconRunning, _("Processing Hugin's batch queue"));
@@ -810,31 +774,36 @@ void BatchFrame::OnButtonRemoveComplete(wxCommandEvent& event)
 
 void BatchFrame::OnButtonRemoveFromList(wxCommandEvent& event)
 {
-    int selIndex = projListBox->GetSelectedIndex();
-    if(selIndex != -1)
+    
+    const HuginBase::UIntSet selected = projListBox->GetSelectedProjects();
+    if(!selected.empty())
     {
-        if(m_batch->GetStatus(selIndex)==Project::RUNNING || m_batch->GetStatus(selIndex)==Project::PAUSED)
+        for (auto i = selected.rbegin(); i != selected.rend(); ++i)
         {
-            wxMessageDialog message(this, _("Cannot remove project in progress.\nDo you want to cancel it?"),
-#ifdef _WIN32
-                                    _("PTBatcherGUI"),
-#else
-                                    wxT(""),
-#endif
-                                    wxYES_NO | wxICON_INFORMATION);
-            if(message.ShowModal()==wxID_YES)
+            const int selIndex = *i;
+            if (m_batch->GetStatus(selIndex) == Project::RUNNING || m_batch->GetStatus(selIndex) == Project::PAUSED)
             {
-                OnButtonSkip(event);
+                wxMessageDialog message(this, _("Cannot remove project in progress.\nDo you want to cancel it?"),
+#ifdef _WIN32
+                    _("PTBatcherGUI"),
+#else
+                    wxT(""),
+#endif
+                    wxYES_NO | wxICON_INFORMATION);
+                if (message.ShowModal() == wxID_YES)
+                {
+                    OnButtonSkip(event);
+                };
             }
-        }
-        else
-        {
-            SetStatusText(wxString::Format(_("Removed project %s"), projListBox->GetSelectedProject().c_str()));
-            projListBox->Deselect(selIndex);
-            projListBox->DeleteItem(selIndex);
-            m_batch->RemoveProjectAtIndex(selIndex);
-            m_batch->SaveTemp();
-        }
+            else
+            {
+                SetStatusText(wxString::Format(_("Removed project %s"), m_batch->GetProject(selIndex)->path.c_str()));
+                projListBox->Deselect(selIndex);
+                projListBox->DeleteItem(selIndex);
+                m_batch->RemoveProjectAtIndex(selIndex);
+                m_batch->SaveTemp();
+            };
+        };
     }
     else
     {
@@ -845,28 +814,31 @@ void BatchFrame::OnButtonRemoveFromList(wxCommandEvent& event)
 
 void BatchFrame::OnButtonReset(wxCommandEvent& event)
 {
-    int selIndex = projListBox->GetSelectedIndex();
-    if(selIndex != -1)
+    const HuginBase::UIntSet selected(projListBox->GetSelectedProjects());
+    if(!selected.empty())
     {
-        if(m_batch->GetStatus(selIndex)==Project::RUNNING || m_batch->GetStatus(selIndex)==Project::PAUSED)
+        for (auto selIndex : selected)
         {
-            wxMessageDialog message(this, _("Cannot reset project in progress.\nDo you want to cancel it?"),
-#ifdef _WIN32
-                                    _("PTBatcherGUI"),
-#else
-                                    wxT(""),
-#endif
-                                    wxYES_NO | wxICON_INFORMATION);
-            if(message.ShowModal()==wxID_YES)
+            if (m_batch->GetStatus(selIndex) == Project::RUNNING || m_batch->GetStatus(selIndex) == Project::PAUSED)
             {
-                OnButtonSkip(event);
+                wxMessageDialog message(this, _("Cannot reset project in progress.\nDo you want to cancel it?"),
+#ifdef _WIN32
+                    _("PTBatcherGUI"),
+#else
+                    wxT(""),
+#endif
+                    wxYES_NO | wxICON_INFORMATION);
+                if (message.ShowModal() == wxID_YES)
+                {
+                    OnButtonSkip(event);
+                }
             }
-        }
-        else
-        {
-            m_batch->SetStatus(selIndex,Project::WAITING);
-            SetStatusText(wxString::Format(_("Reset project %s"), projListBox->GetSelectedProject().c_str()));
-        }
+            else
+            {
+                m_batch->SetStatus(selIndex, Project::WAITING);
+                SetStatusText(wxString::Format(_("Reset project %s"), m_batch->GetProject(selIndex)->path.c_str()));
+            };
+        };
     }
     else
     {
@@ -931,44 +903,47 @@ void BatchFrame::OnButtonSaveBatch(wxCommandEvent& event)
 
 void BatchFrame::OnButtonSkip(wxCommandEvent& event)
 {
-    int selIndex = projListBox->GetSelectedIndex();
-    if(selIndex != -1)
+    const HuginBase::UIntSet selected(projListBox->GetSelectedProjects());
+    if(!selected.empty())
     {
-        if(m_batch->GetStatus(selIndex)==Project::RUNNING
-                ||m_batch->GetStatus(selIndex)==Project::PAUSED)
+        for (auto selIndex : selected)
         {
-            if(m_batch->GetStatus(selIndex)==Project::PAUSED)
+            if (m_batch->GetStatus(selIndex) == Project::RUNNING
+                || m_batch->GetStatus(selIndex) == Project::PAUSED)
             {
-                if(m_batch->GetRunningCount()==1)
+                if (m_batch->GetStatus(selIndex) == Project::PAUSED)
                 {
-                    GetToolBar()->ToggleTool(XRCID("tool_pause"),false);
-                }
-                for(int i=0; i<m_batch->GetRunningCount(); i++)
-                {
-                    if(m_batch->GetStatus(selIndex)==Project::PAUSED
-                            && m_batch->CompareProjectsInLists(i,selIndex))
+                    if (m_batch->GetRunningCount() == 1)
                     {
-                        m_batch->CancelProject(i);
+                        GetToolBar()->ToggleTool(XRCID("tool_pause"), false);
+                    }
+                    for (int i = 0; i < m_batch->GetRunningCount(); i++)
+                    {
+                        if (m_batch->GetStatus(selIndex) == Project::PAUSED
+                            && m_batch->CompareProjectsInLists(i, selIndex))
+                        {
+                            m_batch->CancelProject(i);
+                        }
+                    }
+                }
+                else
+                {
+                    //we go through all running projects to find the one with the same id as chosen one
+                    for (int i = 0; i < m_batch->GetRunningCount(); i++)
+                    {
+                        if (m_batch->GetStatus(selIndex) == Project::RUNNING
+                            && m_batch->CompareProjectsInLists(i, selIndex))
+                        {
+                            m_batch->CancelProject(i);
+                        }
                     }
                 }
             }
             else
             {
-                //we go through all running projects to find the one with the same id as chosen one
-                for(int i=0; i<m_batch->GetRunningCount(); i++)
-                {
-                    if(m_batch->GetStatus(selIndex)==Project::RUNNING
-                            && m_batch->CompareProjectsInLists(i,selIndex))
-                    {
-                        m_batch->CancelProject(i);
-                    }
-                }
-            }
-        }
-        else
-        {
-            m_batch->SetStatus(selIndex,Project::FAILED);
-        }
+                m_batch->SetStatus(selIndex, Project::FAILED);
+            };
+        };
     }
 }
 
@@ -1156,9 +1131,7 @@ void BatchFrame::OnClose(wxCloseEvent& event)
             wxT(""),
 #endif
             wxYES_NO | wxICON_INFORMATION);
-#if wxCHECK_VERSION(3,0,0)
         message.SetYesNoLabels(_("Clear batch queue now"), _("Keep batch queue"));
-#endif
         if (message.ShowModal() == wxID_YES)
         {
             m_batch->ClearBatch();
@@ -1192,9 +1165,6 @@ void BatchFrame::OnClose(wxCloseEvent& event)
     {
         this->GetThread()->Wait();
     };
-#ifndef __WXMSW__
-    delete m_help;
-#endif
     if(m_tray!=NULL)
     {
         delete m_tray;
@@ -1216,7 +1186,7 @@ void BatchFrame::RunBatch()
 {
     if(!IsRunning())
     {
-        SetStatusInformation(_("Starting batch"),true);
+        SetStatusInformation(_("Starting batch"));
         if (m_tray)
         {
             m_tray->SetIcon(m_iconRunning, _("Processing Hugin's batch queue"));
@@ -1231,20 +1201,22 @@ void BatchFrame::SetLocaleAndXRC(wxLocale* locale, wxString xrc)
     m_xrcPrefix = xrc;
 }
 
-void BatchFrame::SwapProject(int index)
+void BatchFrame::SwapProject(int index, bool down)
 {
     if(index>=0 && index<(projListBox->GetItemCount()-1))
     {
         projListBox->SwapProject(index);
         m_batch->SwapProject(index);
-        if(projListBox->GetSelectedIndex()==index)
+        if(down)
         {
-            projListBox->Select(index+1);
+            projListBox->Deselect(index);
+            projListBox->Select(index + 1);
         }
         else
         {
             projListBox->Select(index);
-        }
+            projListBox->Deselect(index + 1);
+        };
     }
 }
 
@@ -1301,7 +1273,7 @@ void BatchFrame::OnBatchFailed(wxCommandEvent& event)
 
 void BatchFrame::OnBatchInformation(wxCommandEvent& e)
 {
-    SetStatusInformation(e.GetString(),true);
+    SetStatusInformation(e.GetString());
     if (m_tray && e.GetInt() == 1)
     {
         // batch finished, reset icon in task bar
@@ -1309,12 +1281,12 @@ void BatchFrame::OnBatchInformation(wxCommandEvent& e)
     };
 };
 
-void BatchFrame::SetStatusInformation(wxString status,bool showBalloon)
+void BatchFrame::SetStatusInformation(wxString status)
 {
     SetStatusText(status);
-    if(m_tray!=NULL && showBalloon)
+    if(m_tray!=NULL)
     {
-#if defined __WXMSW__ && wxUSE_TASKBARICON_BALLOONS && wxCHECK_VERSION(2,9,0)
+#if defined __WXMSW__ && wxUSE_TASKBARICON_BALLOONS 
         m_tray->ShowBalloon(_("PTBatcherGUI"),status,5000,wxICON_INFORMATION);
 #else
 #ifndef __WXMAC__
@@ -1365,15 +1337,9 @@ void BatchFrame::OnMinimize(wxIconizeEvent& e)
     //hide/show window in taskbar when minimizing
     if(m_tray!=NULL)
     {
-#if wxCHECK_VERSION(2,9,0)
         Show(!e.IsIconized());
         //switch off verbose output if PTBatcherGUI is in tray/taskbar
         if(e.IsIconized())
-#else
-        Show(!e.Iconized());
-        //switch off verbose output if PTBatcherGUI is in tray/taskbar
-        if(e.Iconized())
-#endif
         {
             m_batch->verbose=false;
         }
@@ -1386,11 +1352,7 @@ void BatchFrame::OnMinimize(wxIconizeEvent& e)
     }
     else //don't hide window if no tray icon
     {
-#if wxCHECK_VERSION(2,9,0)
         if (!e.IsIconized())
-#else
-        if (!e.Iconized())
-#endif
         {
             // window is restored, update progress indicators
             UpdateTaskBarProgressBar();
@@ -1407,17 +1369,17 @@ void BatchFrame::UpdateBatchVerboseStatus()
 
 void BatchFrame::OnRefillListBox(wxCommandEvent& event)
 {
-    int index=projListBox->GetSelectedIndex();
-    int id=-1;
-    if(index!=-1)
+    const HuginBase::UIntSet selected = projListBox->GetSelectedProjects();
+    std::set<long> selectedID;
+    for (auto index : selected)
     {
-        id=projListBox->GetProjectId(index);
+        selectedID.insert(m_batch->GetProject(index)->id);
     };
     projListBox->DeleteAllItems();
     projListBox->Fill(m_batch);
-    if(id!=-1)
+    for(auto id:selectedID)
     {
-        index=projListBox->GetIndex(id);
+        int index=projListBox->GetIndex(id);
         if(index!=-1)
         {
             projListBox->Select(index);
