@@ -87,9 +87,7 @@ FindPanoDialog::FindPanoDialog(BatchFrame* batchframe, wxString xrcPrefix)
     m_button_send=XRCCTRL(*this,"find_pano_add_queue",wxButton);
     m_button_close=XRCCTRL(*this,"find_pano_close",wxButton);
     m_textctrl_dir=XRCCTRL(*this,"find_pano_dir",wxTextCtrl);
-#if wxCHECK_VERSION(2,9,3)
     m_textctrl_dir->AutoCompleteDirectories();
-#endif
     m_cb_subdir=XRCCTRL(*this,"find_pano_subdir",wxCheckBox);
     m_statustext=XRCCTRL(*this,"find_pano_label",wxStaticText);
     m_list_pano=XRCCTRL(*this,"find_pano_list",wxCheckListBox);
@@ -441,13 +439,8 @@ void FindPanoDialog::OnSplitPanos(wxCommandEvent &e)
                     m_panos.insert(m_panos.begin() + selectedPano + 1, newSubPano);
                     // update pano list
                     m_list_pano->SetString(selectedPano, m_panos[selectedPano]->GetItemString(m_start_dir));
-#if wxCHECK_VERSION(3,0,0)
                     int newItem = m_list_pano->Insert(m_panos[selectedPano + 1]->GetItemString(m_start_dir), selectedPano + 1);
                     m_list_pano->Check(newItem, true);
-#else
-                    m_list_pano->Insert(m_panos[selectedPano + 1]->GetItemString(m_start_dir), selectedPano + 1);
-                    m_list_pano->Check(selectedPano + 1, true);
-#endif
                     // update display
                     wxCommandEvent dummy;
                     OnSelectPossiblePano(dummy);
