@@ -2,7 +2,7 @@
 /** @file nona/RemappedPanoImage.h
  *
  *  Contains functions to transform whole images.
- *  Can use PTools::Transform or PT::SpaceTransform for the calculations
+ *  Can use PTools::Transform or PanoCommand::SpaceTransform for the calculations
  *
  *  @author Pablo d'Angelo <pablo.dangelo@web.de>
  *
@@ -19,25 +19,32 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #ifndef _NONA_REMAPPEDPANOIMAGE_H
 #define _NONA_REMAPPEDPANOIMAGE_H
 
-#include <vigra/impex.hxx>
+#include <vigra/imageinfo.hxx>
+#include <vigra/initimage.hxx>
+#include <vigra/copyimage.hxx>
 #include <vigra/flatmorphology.hxx>
 #include <vigra_ext/ROIImage.h>
 
-#include <appbase/ProgressDisplayOld.h>
+#include <appbase/ProgressDisplay.h>
+#include <nona/StitcherOptions.h>
 
 #include <panodata/SrcPanoImage.h>
 #include <panodata/Mask.h>
 #include <panodata/PanoramaOptions.h>
 #include <panotools/PanoToolsInterface.h>
 
+// default values for exposure cutoff
+#define NONA_DEFAULT_EXPOSURE_LOWER_CUTOFF 1/255.0f
+#define NONA_DEFAULT_EXPOSURE_UPPER_CUTOFF 250/255.0f
+
 
 namespace HuginBase {
 namespace Nona {
@@ -96,7 +103,7 @@ class RemappedPanoImage : public vigra_ext::ROIImage<RemapImage, AlphaImage>
          *
          *  the actual remapping is done by the remapImage() function.
          */
-        RemappedPanoImage()
+        RemappedPanoImage() : m_advancedOptions()
         {};
 
         
@@ -105,25 +112,10 @@ class RemappedPanoImage : public vigra_ext::ROIImage<RemapImage, AlphaImage>
         void setPanoImage(const SrcPanoImage & src,
                           const PanoramaOptions & dest,
                           vigra::Rect2D roi);
-
-    //    /** set a new image or panorama options
-    //     *
-    //     *  This is needed before any of the remap functions can be used.
-    //     *
-    //     *  calculates bounding box, and outline
-    //     */
-    //    void setPanoImage(const vigra::Size2D & srcSize,
-    //                      const PT::VariableMap & srcVars,
-    //                      PT::Lens::LensProjectionFormat srcProj,
-    //                      const PT::PanoImage & img,
-    //                      const vigra::Diff2D &destSize,
-    //                      PT::PanoramaOptions::ProjectionFormat destProj,
-    //                      double destHFOV);
-    //    
-    //    ///
-    //    void setPanoImage(const PT::Panorama & pano, unsigned int imgNr,
-    //                      vigra::Size2D srcSize, const PT::PanoramaOptions & opts);
-        
+        void setAdvancedOptions(const AdvancedOptions& advancedOptions)
+        {
+            m_advancedOptions = advancedOptions;
+        };
 
     public:
         /** calculate distance map. pixels contain distance from image center
@@ -147,7 +139,7 @@ class RemappedPanoImage : public vigra_ext::ROIImage<RemapImage, AlphaImage>
         template <class ImgIter, class ImgAccessor>
         void remapImage(vigra::triple<ImgIter, ImgIter, ImgAccessor> srcImg,
                         vigra_ext::Interpolator interpol,
-                        AppBase::MultiProgressDisplay & progress);
+                        AppBase::ProgressDisplay* progress, bool singleThreaded = false);
 
 
         /** remap a image, with alpha channel */
@@ -156,7 +148,7 @@ class RemappedPanoImage : public vigra_ext::ROIImage<RemapImage, AlphaImage>
         void remapImage(vigra::triple<ImgIter, ImgIter, ImgAccessor> srcImg,
                         std::pair<AlphaIter, AlphaAccessor> alphaImg,
                         vigra_ext::Interpolator interp,
-                        AppBase::MultiProgressDisplay & progress);
+                        AppBase::ProgressDisplay* progress, bool singleThreaded = false);
         
         
     public:
@@ -167,6 +159,7 @@ class RemappedPanoImage : public vigra_ext::ROIImage<RemapImage, AlphaImage>
         SrcPanoImage m_srcImg;
         PanoramaOptions m_destImg;
         PTools::Transform m_transf;
+        AdvancedOptions m_advancedOptions;
 
 };
 
@@ -182,7 +175,7 @@ void remapImage(SrcImgType & srcImg,
                 const PanoramaOptions & dest,
                 vigra::Rect2D outputRect,
                 RemappedPanoImage<DestImgType, MaskImgType> & remapped,
-                AppBase::MultiProgressDisplay & progress);
+                AppBase::ProgressDisplay* progress);
 
 
 } // namespace
@@ -198,12 +191,11 @@ void remapImage(SrcImgType & srcImg,
 #include <vigra_ext/ImageTransforms.h>
 #include <vigra_ext/ImageTransformsGPU.h>
 
-#ifdef DEBUG
-#define DEBUG_REMAP 1
-#endif
+// #define DEBUG_REMAP 1
 
 #ifdef DEBUG_REMAP
-#ifdef WIN32
+#include <vigra/impex.hxx> // for vigra::exportImage()
+#ifdef _WIN32
 #define DEBUG_FILE_PREFIX "C:/temp/"
 #else
 #define DEBUG_FILE_PREFIX "/tmp/"
@@ -246,11 +238,11 @@ void RemappedPanoImage<RemapImage,AlphaImage>::setPanoImage(const SrcPanoImage &
  */
 template <class RemapImage, class AlphaImage>
 void RemappedPanoImage<RemapImage,AlphaImage>::setPanoImage(const vigra::Size2D & srcSize,
-                  const PT::VariableMap & srcVars,
-                  PT::Lens::LensProjectionFormat srcProj,
-                  const PT::PanoImage & img,
+                  const PanoCommand::VariableMap & srcVars,
+                  PanoCommand::Lens::LensProjectionFormat srcProj,
+                  const PanoCommand::PanoImage & img,
                   const vigra::Diff2D &destSize,
-                  PT::PanoramaOptions::ProjectionFormat destProj,
+                  HuginBase::PanoramaOptions::ProjectionFormat destProj,
                   double destHFOV)
 {
     m_srcSize = srcSize;
@@ -290,10 +282,10 @@ void RemappedPanoImage<RemapImage,AlphaImage>::setPanoImage(const vigra::Size2D
 }
 
 template <class RemapImage, class AlphaImage>
-void RemappedPanoImage<RemapImage,AlphaImage>::setPanoImage(const PT::Panorama & pano, unsigned int imgNr,
-                  vigra::Size2D srcSize, const PT::PanoramaOptions & opts)
+void RemappedPanoImage<RemapImage,AlphaImage>::setPanoImage(const HuginBase::Panorama & pano, unsigned int imgNr,
+                  vigra::Size2D srcSize, const HuginBase::PanoramaOptions & opts)
 {
-    const PT::PanoImage & img = pano.getImage(imgNr);
+    const PanoCommand::PanoImage & img = pano.getImage(imgNr);
 
     m_srcSize = srcSize;
     m_srcOrigSize.x = img.getWidth();
@@ -389,13 +381,13 @@ void RemappedPanoImage<RemapImage,AlphaImage>::calcAlpha()
     int ystart = Base::boundingBox().top();
     int yend   = Base::boundingBox().bottom();
 
-// DGSW FIXME - Unreferenced
-//		int interpolHalfWidth=0;
-    // create dist y iterator
-    typename AlphaImage::Iterator yalpha(Base::m_mask.upperLeft());
     // loop over the image and transform
-    for(int y=ystart; y < yend; ++y, ++yalpha.y)
+#pragma omp parallel for schedule(dynamic, 10)
+    for(int y=ystart; y < yend; ++y)
     {
+        // create dist y iterator
+        typename AlphaImage::Iterator yalpha(Base::m_mask.upperLeft());
+        yalpha.y += y - ystart;
         // create x iterators
         typename AlphaImage::Iterator xalpha(yalpha);
         for(int x=xstart; x < xend; ++x, ++xalpha.x)
@@ -425,7 +417,7 @@ template<class RemapImage, class AlphaImage>
 template<class ImgIter, class ImgAccessor>
 void RemappedPanoImage<RemapImage,AlphaImage>::remapImage(vigra::triple<ImgIter, ImgIter, ImgAccessor> srcImg,
                                                           vigra_ext::Interpolator interpol,
-                                                          AppBase::MultiProgressDisplay & progress)
+                                                          AppBase::ProgressDisplay* progress, bool singleThreaded)
 {
 
     //        std::ostringstream msg;
@@ -440,14 +432,15 @@ void RemappedPanoImage<RemapImage,AlphaImage>::remapImage(vigra::triple<ImgIter,
     vigra::Diff2D srcImgSize = srcImg.second - srcImg.first;
 
     vigra::Size2D expectedSize = m_srcImg.getSize();
-    if (useGPU) {
+    if (useGPU)
+    {
         const int r = expectedSize.width() % 8;
         if (r != 0) expectedSize += vigra::Diff2D(8 - r, 0);
     }
 
     DEBUG_DEBUG("srcImgSize: " << srcImgSize << " m_srcImgSize: " << m_srcImg.getSize());
     vigra_precondition(srcImgSize == expectedSize, 
-                       "RemappedPanoImage<RemapImage,AlphaImage>::remapImage(): image sizes not consistent");
+                       "RemappedPanoImage<RemapImage,AlphaImage>::remapImage(): image unexpectedly changed dimensions.");
 
     typedef typename ImgAccessor::value_type input_value_type;
     typedef typename vigra_ext::ValueTypeTraits<input_value_type>::value_type input_component_type;
@@ -460,7 +453,10 @@ void RemappedPanoImage<RemapImage,AlphaImage>::remapImage(vigra::triple<ImgIter,
     if (m_destImg.outputMode == PanoramaOptions::OUTPUT_LDR) {
         // select exposure and response curve for LDR output
         std::vector<double> outLut;
-        vigra_ext::EMoR::createEMoRLUT(m_destImg.outputEMoRParams, outLut);
+        if (!m_destImg.outputEMoRParams.empty())
+        {
+            vigra_ext::EMoR::createEMoRLUT(m_destImg.outputEMoRParams, outLut);
+        };
         double maxVal = vigra_ext::LUTTraits<input_value_type>::max();
         if (m_destImg.outputPixelType.size() > 0) {
             maxVal = vigra_ext::getMaxValForPixelType(m_destImg.outputPixelType);
@@ -469,11 +465,10 @@ void RemappedPanoImage<RemapImage,AlphaImage>::remapImage(vigra::triple<ImgIter,
         invResponse.setOutput(1.0/pow(2.0,m_destImg.outputExposureValue), outLut,
                               maxVal);
     } else {
-        invResponse.setHDROutput();
+        invResponse.setHDROutput(true,1.0/pow(2.0,m_destImg.outputExposureValue));
     }
 
-
-    if ((m_srcImg.hasActiveMasks()) || (m_srcImg.getCropMode() != SrcPanoImage::NO_CROP))
+    if ((m_srcImg.hasActiveMasks()) || (m_srcImg.getCropMode() != SrcPanoImage::NO_CROP) || Nona::GetAdvancedOption(m_advancedOptions, "maskClipExposure", false))
     {
         // need to create and additional alpha image for the crop mask...
         // not very efficient during the remapping phase, but works.
@@ -529,6 +524,12 @@ void RemappedPanoImage<RemapImage,AlphaImage>::remapImage(vigra::triple<ImgIter,
         }
         if(m_srcImg.hasActiveMasks())
             vigra_ext::applyMask(vigra::destImageRange(alpha), m_srcImg.getActiveMasks());
+        if (Nona::GetAdvancedOption(m_advancedOptions, "maskClipExposure", false))
+        {
+            const float lowerCutoff = Nona::GetAdvancedOption(m_advancedOptions, "maskClipExposureLowerCutoff", NONA_DEFAULT_EXPOSURE_LOWER_CUTOFF);
+            const float upperCutoff = Nona::GetAdvancedOption(m_advancedOptions, "maskClipExposureUpperCutoff", NONA_DEFAULT_EXPOSURE_UPPER_CUTOFF);
+            vigra_ext::applyExposureClipMask(srcImg, vigra::destImageRange(alpha), lowerCutoff, upperCutoff);
+        };
         if (useGPU) {
             transformImageAlphaGPU(srcImg,
                                    vigra::srcImage(alpha),
@@ -550,7 +551,8 @@ void RemappedPanoImage<RemapImage,AlphaImage>::remapImage(vigra::triple<ImgIter,
                                 invResponse,
                                 m_srcImg.horizontalWarpNeeded(),
                                 interpol,
-                                progress);
+                                progress,
+                                singleThreaded);
         }
     } else {
         if (useGPU) {
@@ -593,7 +595,8 @@ void RemappedPanoImage<RemapImage,AlphaImage>::remapImage(vigra::triple<ImgIter,
                            invResponse,
                            m_srcImg.horizontalWarpNeeded(),
                            interpol,
-                           progress);
+                           progress,
+                           singleThreaded);
         }
     }
 }
@@ -607,23 +610,25 @@ template<class ImgIter, class ImgAccessor,
 void RemappedPanoImage<RemapImage,AlphaImage>::remapImage(vigra::triple<ImgIter, ImgIter, ImgAccessor> srcImg,
                                                           std::pair<AlphaIter, AlphaAccessor> alphaImg,
                                                           vigra_ext::Interpolator interp,
-                                                          AppBase::MultiProgressDisplay & progress)
+                                                          AppBase::ProgressDisplay* progress, bool singleThreaded)
 {
     const bool useGPU = m_destImg.remapUsingGPU;
 
     if (Base::boundingBox().isEmpty())
         return;
 
+    progress->setMessage("remapping", hugin_utils::stripPath(m_srcImg.getFilename()));
+
     vigra::Diff2D srcImgSize = srcImg.second - srcImg.first;
 
     vigra::Size2D expectedSize = m_srcImg.getSize();
-    if (useGPU) {
+    if (useGPU)
+    {
         const int r = expectedSize.width() % 8;
         if (r != 0) expectedSize += vigra::Diff2D(8 - r, 0);
     }
-
-    vigra_precondition(srcImgSize == expectedSize, 
-                       "RemappedPanoImage<RemapImage,AlphaImage>::remapImage(): image sizes not consistent");
+    vigra_precondition(srcImgSize == expectedSize,
+                       "RemappedPanoImage<RemapImage,AlphaImage>::remapImage(): image unexpectedly changed dimensions.");
 
     typedef typename ImgAccessor::value_type input_value_type;
     typedef typename vigra_ext::ValueTypeTraits<input_value_type>::value_type input_component_type;
@@ -640,15 +645,18 @@ void RemappedPanoImage<RemapImage,AlphaImage>::remapImage(vigra::triple<ImgIter,
         if (m_destImg.outputPixelType.size() > 0) {
             maxVal = vigra_ext::getMaxValForPixelType(m_destImg.outputPixelType);
         }
-        vigra_ext::EMoR::createEMoRLUT(m_destImg.outputEMoRParams, outLut);
-		vigra_ext::enforceMonotonicity(outLut);
+        if (!m_destImg.outputEMoRParams.empty())
+        {
+            vigra_ext::EMoR::createEMoRLUT(m_destImg.outputEMoRParams, outLut);
+            vigra_ext::enforceMonotonicity(outLut);
+        };
         invResponse.setOutput(1.0/pow(2.0,m_destImg.outputExposureValue), outLut,
                               maxVal);
     } else {
-        invResponse.setHDROutput();
+        invResponse.setHDROutput(true,1.0/pow(2.0,m_destImg.outputExposureValue));
     }
 
-    if (((m_srcImg.hasActiveMasks()) || (m_srcImg.getCropMode() != SrcPanoImage::NO_CROP))) {
+    if ((m_srcImg.hasActiveMasks()) || (m_srcImg.getCropMode() != SrcPanoImage::NO_CROP) || Nona::GetAdvancedOption(m_advancedOptions, "maskClipExposure", false)) {
         vigra::BImage alpha(srcImgSize);
         vigra::Rect2D cR = m_srcImg.getCropRect();
         switch (m_srcImg.getCropMode()) {
@@ -693,7 +701,12 @@ void RemappedPanoImage<RemapImage,AlphaImage>::remapImage(vigra::triple<ImgIter,
         }
         if(m_srcImg.hasActiveMasks())
             vigra_ext::applyMask(vigra::destImageRange(alpha), m_srcImg.getActiveMasks());
-
+        if (Nona::GetAdvancedOption(m_advancedOptions, "maskClipExposure", false))
+        {
+            const float lowerCutoff = Nona::GetAdvancedOption(m_advancedOptions, "maskClipExposureLowerCutoff", NONA_DEFAULT_EXPOSURE_LOWER_CUTOFF);
+            const float upperCutoff = Nona::GetAdvancedOption(m_advancedOptions, "maskClipExposureUpperCutoff", NONA_DEFAULT_EXPOSURE_UPPER_CUTOFF);
+            vigra_ext::applyExposureClipMask(srcImg, vigra::destImageRange(alpha), lowerCutoff, upperCutoff);
+        };
         if (useGPU) {
             vigra_ext::transformImageAlphaGPU(srcImg,
                                               vigra::srcImage(alpha),
@@ -715,7 +728,8 @@ void RemappedPanoImage<RemapImage,AlphaImage>::remapImage(vigra::triple<ImgIter,
                                            invResponse,
                                            m_srcImg.horizontalWarpNeeded(),
                                            interp,
-                                           progress);
+                                           progress,
+                                           singleThreaded);
         }
     } else {
         if (useGPU) {
@@ -740,7 +754,8 @@ void RemappedPanoImage<RemapImage,AlphaImage>::remapImage(vigra::triple<ImgIter,
                                            invResponse,
                                            m_srcImg.horizontalWarpNeeded(),
                                            interp,
-                                           progress);
+                                           progress,
+                                           singleThreaded);
         }
     }
 }
@@ -761,13 +776,8 @@ void remapImage(SrcImgType & srcImg,
                 vigra::Rect2D outputROI,
 //                vigra_ext::Interpolator interpolator,
                 RemappedPanoImage<DestImgType, MaskImgType> & remapped,
-                AppBase::MultiProgressDisplay & progress)
+                AppBase::ProgressDisplay* progress)
 {
-    typedef typename SrcImgType::value_type SrcPixelType;
-    typedef typename DestImgType::value_type DestPixelType;
-
-    typedef typename vigra::NumericTraits<SrcPixelType>::RealPromote RSrcPixelType;
-
 #ifdef DEBUG_REMAP
     {
         vigra::ImageExportInfo exi( DEBUG_FILE_PREFIX "hugin03_BeforeRemap.tif");
@@ -781,7 +791,7 @@ void remapImage(SrcImgType & srcImg,
     }
 #endif
 
-    progress.setMessage(std::string("remapping ") + hugin_utils::stripPath(src.getFilename()));
+    progress->setMessage("remapping", hugin_utils::stripPath(src.getFilename()));
     // set pano image
     DEBUG_DEBUG("setting src image with size: " << src.getSize());
     remapped.setPanoImage(src, dest, outputROI);
